#include <stdio.h>
#include <getopt.h>

#include "base.h"
#include "proc.h"
#include "gui.h"
#include "ncps.h"
#include "parser.h"
#include "tree.h"

static void cleanup(int nil)
{
    clear_all_screen();
    endwin();
    fprintf(stderr, "Terminated on signal %i.\n", nil);
    exit(0);
}


static void usage(void)
{
    printf("Usage: ncps [options]\n"
	   "\t-a, --all \t: display all processes\n"
	   "\t-U <user1,user2,...>, --user <user1,user2,...> : display processes owned by the specified list of users\n"
	   "\t-u, --detailed \t: detailed output\n"
	   "\t-t --tree \t: starts in tree mode\n "
	   "\t-h, --help \t: display this help.\n");
}

int main(int argc, char *argv[])
{
    int check = 1;
    int tmp;
    
    killer_t killer;
    killer_t *chow;
    int c;
    int opt_a = 0, opt_t = 0, opt_u = 0;
    char *homedir;
    uid_t *uidlist = NULL;
    char *path;
    char *users = NULL;

    int i, j;

    
    for(i = 1; i < FIELDS; i++) {
      field_op_[i] = 1;
    }

    field_op_[FUSER] = 0;
    field_op_[FCPU] = 0;
    field_op_[FMEM] = 0;

    fields_ = FIELDS - 3;

    signal(SIGILL, cleanup);
    signal(SIGTERM, cleanup);
    signal(SIGSEGV, cleanup);
    signal(SIGABRT, cleanup);
    signal(SIGHUP, cleanup);

    /*  printf("initscr ended\n"); */

    for (;;) {
	static struct option long_options[] =
	{
	    {"all", 0, 0, 'a'},
	    {"detailed", 0, 0, 'u'},
	    {"user", 1, 0, 'U'},
	    {"tree", 0, 0, 't'},
	    {"help", 0, 0, 'h'},
	    {"version", 0, 0, 'v'},
	    {0, 0, 0, 0}
	};

	c = getopt_long(argc, argv, "auU:thv", long_options, NULL);
	if (c == -1)
	    break;

	switch (c) {
	case 'a':
	  if(!opt_u) {
	    field_op_[FUSER] = 1;
	    fields_++;
	  }
	  opt_a = 1;
	  break;
	  
	case 'u':
	  for(i = 0; i < FIELDS; i++) {
	    field_op_[i] = 1;
	  }
	  fields_ = FIELDS;
	  break;
	  
	case 't':
	    opt_t = 1;
	    break;

	case 'U':
	  if(!opt_a) {
	    field_op_[FUSER] = 1;
	    fields_++;
	  }
	  opt_u = 1;
	  users = (char *) malloc( (strlen(optarg) + 1) * sizeof(char));
	  strcpy(users, optarg);
	  break;
	    
	case 'h':
	  printf("ncps: ncurses based process manager. Version %s\n",
		 VERSION);
	  usage();
	  exit(0);
	  
	case 'v':
	  printf("ncps: ncurses based process manager. Version %s\n",
		  VERSION);
	  exit(0);

	case '?':
	    usage();
	    exit(1);

	default:
	    printf("?? getopt returned character code 0%o ??\n", c);
	    exit(1);
	}
    }

    /* both -a and -u cannot be present */
    if (opt_a && opt_u) {
	fprintf(stderr, "ncps: Options 'all' and 'user' are mutually "
		"incompatible.\n");
	exit(1);
    }
    if (opt_a)
      killer.uidlist = NULL;
    else if (opt_u) {
      killer.uidlist = (unsigned int *) parse_usernames(users, (int *) uidlist);
    }
    else {
      killer.uidlist = (uid_t *) malloc(2 * sizeof(uid_t));
      killer.uidlist[0] = 1;
      killer.uidlist[1]= getuid();	/* default: current user */
    }

    t_view = opt_t;
	
    avfields_ = (int *) malloc(fields_ * sizeof(int));
    j = 0;
    for(i = 1; i < FIELDS; i++) {
      if(field_op_[i]) {
	avfields_[j] = i;
	j++;
      }
    }

#if !DEBUG
    initscr();
    
    check_scr_size();
    
    curs_set(0);
    keypad(stdscr, TRUE);
    cbreak();
    noecho();

    homedir = getenv ("HOME");
    path = (char *) malloc((strlen(homedir) + 9) * sizeof(char));
    sprintf(path, "%s/.ncpsrc", homedir);
    
    parse_rc_color(path, color_config);
    
    free(path);

    start_color();   
    
    for(i = 1; i < COLOR_OPTIONS; i++) {
      init_pair(i, color_config[i].foreground, 
		color_config[i].background);
    }    

  
#endif

   
    template_size = PT_SIZE;
    selection_template = (int *) malloc(template_size * sizeof(int));
    selection_template[0] = -1;
    
    chow = &killer;

    initplist(chow);

    chow->t_count = 0;      
    make_tree(chow);

#if DEBUG    
    printf("tree planted\n"); 
#endif

#if !DEBUG
    clear_all_screen();
#endif

    chow = &killer;

#if DEBUG
    check = 0;
#endif

    while (check) {

	free(selection_template);
	refresh_stemplate(chow);

#if DEBUG
	printf("getting choice\n");
#endif
	
	tmp = getchoice(chow);

#if DEBUG
        printf("choice obtained\n");
#endif
	
	if (tmp == QUIT)
	  check = 0;
	else if (tmp == REINIT) {
	    endwin();
	    free_list(chow->pidlist, chow->count);
	    free_list(chow->trunk, chow->t_count);
	    free_branch(chow->ptree);
	    
	    initplist(chow);
	    
	    chow->t_count = 0;
	    make_tree(chow);
	    
	    initscr();
	    curs_set(0);
	    clear_all_screen();
	} 
    }

#if !DEBUG
    clear_all_screen();
    endwin();
#endif

    return 1;
}				/* main */





			   


